AnatomyTagger
=============

Entity mention recognition system for anatomical entities.

## Installation

The tagger depends on the NERsuite toolkit (http://nersuite.nlplab.org).
If you don't have a recent version of NERsuite installed, please follow the
[installation instructions below](#installing-related-tools) first.

To set up the tagger itself, simply unpack it and run `setup.sh`
    
    tar xvzf anatomytagger-1.0.0.tar.gz
    cd anatomytagger-1.0.0
    ./setup.sh

The tagger can then be run as

    ./anatomytagger

in the directory it was unpacked into.

## Basic usage

The following sections illustrate basic usage using a small example
dataset provided with the tagger. The data is found in the directory
`data/example` and provided in the standoff format used by the brat
annotation tool (see http://brat.nlplab.org/standoff.html).

Note that you can get general help by running

    ./anatomytagger -h

and detailed help for using

    ./anatomytagger COMMAND -h

where `COMMAND` is one of `learn`, `tag`, `eval`, or `convert`.

### Training

To train a model with default parameters on the example data, run

    ./anatomytagger learn data/example/train/

the model will be stored as `default.model`. To use more than one
model, specify the model file name with the `-m` argument.
    
### Tagging

To tag a single example text document with the previously trained
default model, run

    ./anatomytagger tag data/example/test/PMID-12813373.txt

tagging a single document at a time is fairly slow due to startup
costs. To tag multiple example documents at once, create an output
directory first, and then use `tag` with the `-o` argument, e.g.

    mkdir example-output
    ./anatomytagger tag data/example/test -o example-output/


### Formats and conversion

Outputs are stored by default in the NERsuite variant of the CoNLL
format. For other formats, use the `-f` argument, e.g.

    mkdir example-standoff-output
    ./anatomytagger tag data/example/test -f standoff -o example-standoff-output/

Note that for `standoff`, the output contains a `.txt` and `.ann` file
for each input document.

The `convert` command can be used to convert between supported
formats.  For example, to convert the example test documents into the
NERsuite format:

    mkdir example-test-nersuite
    ./anatomytagger convert -f nersuite -o example-test-nersuite/ data/example/test/

### Evaluation

The example standoff outputs created as above can be evaluated against
reference ("gold standard") annotations using a script in the tagger
`tools/` directory

    ./tools/standoff/evalso.py data/example/test/ example-standoff-output/

alternatively, the `eval` command can be used to combine training,
prediction, and evaluation:

    ./anatomytagger eval data/example/train/ -t data/example/test/

### Parameter selection

The `eval` command can be used to evaluate different parameter
combinations, e.g. as follows for the L2 regularization parameter

    ./anatomytagger eval -c '{0.25,0.5,1,2,4}' data/example/train/ -t data/example/devel/ | tee devel-test-results.txt

and then find the best parameters for various evaluation criteria with

    ./tools/parameters/best-parameters.py devel-test-results.txt 

selected parameters can be then applied with `learn`, `tag` and
`eval`, e.g.

    ./anatomytagger eval -c2 4 data/example/train/ -t data/example/test/

### Running as web service

To start the tagger as a web service, run

    ./anatomytagger tag -S

open the URL output by the tagger for usage information.

## Advanced usage

### Advanced parameter selection

The L2 regularization parameter and the label bias parameter settings
for `eval` support parameter ranges with a bash shell-like syntax, and
a limited range of mathematical operations. For example, to evaluate
performance with the parameters {2<sup>-5</sup>, 2<sup>-4</sup>, ...,
2<sup>5</sup>} on the AnEM corpus development set, use

    ./anatomytagger eval -c2 '2**{-5..5}' data/AnEM-1.0.4/standoff/train/ -t data/AnEM-1.0.4/standoff/devel | tee parameter-selection-results.txt
    ./tools/parameters/best-parameters.py parameter-selection-results.txt | cut -f 1,4 | egrep exact

(this should give c2 0.031250 = 2<sup>-5</sup>, f 53.7%; cf. [Ohta et al. DSSD'12](http://nactem.ac.uk/anatomy/docs/ohta2012opendomain.pdf))

The label bias parameter additionally supports shell wildcard-style
expansion, so that e.g. `B-*` expands to all tags beginning with `B-`.
For example,

    ./anatomytagger eval -c2 '2**{-10..5}' -b 'B-*:{-5..5}' data/AnatEM-1.0.2/standoff/train/ -t data/AnatEM-1.0.2/standoff/devel/ | tee parameter-selection-results.txt
    ./tools/parameters/best-parameters.py devel-results-AnatEM-multi-plain.txt | cut -f 1,2,4 | egrep right

(this should give c2 0.001953, b B-*:2.0, f 81.2%; cf. Pyysalo et al. Bioinformatics 2013)

## Installing related tools

The tagger depends on the NERsuite toolkit, which in turn builds on
the CRFsuite implementation of conditional random fields, which
further requires the libLBFGS library. If these tools are not
previously installed on your system, follow the instructions below
to set them up.

### System-wide installation

To perform a *system-wide* installation of libLBFGS, CRFsuite, and
NERsuite, follow the instructions in this section. Note that this
requires the ability to run sudo. If you cannot sudo or prefer to
install the tools only locally, see the next section.

#### libLBFGS

Download the latest version of libLBFGS from
http://www.chokkan.org/software/liblbfgs/. Example:

    wget https://github.com/downloads/chokkan/liblbfgs/liblbfgs-1.10.tar.gz

(The following libLBFGS installation commands assume version 1.10. If
your version differs, replace "1.10" with the relevant version
string.)

Unpack and enter the libLBFGS directory

    tar xzf liblbfgs-1.10.tar.gz
    cd liblbfgs-1.10/

Configure and make the library

    ./configure
    make

Install for all users (requires password)

    sudo make install

Next, install CRFsuite by following the instructions in the next
section.

#### CRFsuite

Note: the following assumes that libLBFGS has been installed according
to the instructions in the previous section.

Download the latest version of CRFsuite from
https://github.com/priancho/crfsuite. Example:

    wget https://github.com/priancho/crfsuite/archive/master.zip -O crfsuite-master.zip

Unpack and enter the CRFsuite directory

    unzip crfsuite-master.zip 
    cd crfsuite-master/

Configure and make

    ./autogen.sh
    ./configure 
    make

Install for all users (requires password)

    sudo make install

Next, install NERsuite by following the instructions in the next
section.

#### NERsuite

Note: the following assumes the CRFsuite has been installed according
to the instructions in the previous section.

Download the latest version of NERsuite from
https://github.com/spyysalo/nersuite. Example:

    wget https://github.com/spyysalo/nersuite/archive/master.zip -O nersuite-master.zip

Unpack and enter the NERsuite directory

    unzip nersuite-master.zip 
    cd nersuite-master

Configure and make

    ./configure
    make

Install for all users (requires password)

    sudo make install

Finally, test the setup by following the instructions in the next
section.

#### Testing your installation

After installing the tools according to the instructions in the
following sections, you should be able to use CRFsuite and NERsuite
from the command line. To perform a minimal test of the installations,
try invoking the tools as follows to show help:

    crfsuite -h
    nersuite -h

if these commands show brief command-line usage instructions, the
tools are likely installed correctly. If instead you see an error
message, see the troubleshooting section below.

#### Troubleshooting

If you see the message "command not found" (or similar) when trying
to run either CRFsuite and NERsuite, make sure that the directory
to which the programs were installed is included in your path. For
example, if you have performed a local installation and are using
the bash shell, try running

    export PATH=$PATH:$HOME/local/bin

before invoking crfsuite and nersuite.

If you see the message "error while loading shared libraries" when
trying to run either CRFsuite or NERsuite, run ldconfig to create
links to the shared libraries as root

    sudo ldconfig

if this does not work, try adding the directory where the libraries
were installed (by default /usr/local/lib) to your LD_LIBRARY_PATH
environment variable

    export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/usr/local/lib

if this does not work, check that the libraries are in fact found in
the expected directory and try reinstalling if not.

### Local installation

To perform a local installation of libLBFGS, CRFsuite, and NERsuite,
follow the instructions in this section.

The tools will be installed in the local/ subdirectory of your home
directory. To install in an alternative location, simply replace
"~/local" with the desired directory in the commands below.

#### libLBFGS

Download the latest version of libLBFGS from
http://www.chokkan.org/software/liblbfgs/ . As of this writing, the
latest version can be downloaded as follows:

    wget https://github.com/downloads/chokkan/liblbfgs/liblbfgs-1.10.tar.gz

(The following libLBFGS installation commands assume version 1.10. If
your version differs, replace "1.10" with the relevant version
string.)

Unpack and enter the libLBFGS directory

    tar xzf liblbfgs-1.10.tar.gz
    cd liblbfgs-1.10/

Configure for local installation. Here, "$HOME/local" is the base
directory under which the tools will be installed. To install in some
other directory, replace the string "$HOME/local" with that directory
in all instructions below.
    
    ./configure --prefix=$HOME/local

Make and install the library locally

    make
    make install

Next, install CRFsuite by following the instructions in the next
section.

#### CRFsuite

Note: the following assumes that libLBFGS has been installed according
to the instructions in the previous section.

Download the latest version of CRFsuite from
https://github.com/priancho/crfsuite. Example:

    wget https://github.com/priancho/crfsuite/archive/master.zip -O crfsuite-master.zip

Unpack and enter the CRFsuite directory

    unzip crfsuite-master.zip 
    cd crfsuite-master/

Configure for local installation.

    ./autogen.sh
    ./configure --prefix=$HOME/local --with-liblbfgs=$HOME/local

Make and install locally

    make
    make install

Next, install NERsuite by following the instructions in the next
section.

#### NERsuite

Note: the following assumes the CRFsuite has been installed according
to the instructions in the previous section.

Download the latest version of NERsuite from
https://github.com/spyysalo/nersuite. Example:

    wget https://github.com/spyysalo/nersuite/archive/master.zip -O nersuite-master.zip

Unpack and enter the NERsuite directory

    unzip nersuite-master.zip 
    cd nersuite-master

Configure for local installation

    ./configure --prefix=$HOME/local --with-liblbfgs=$HOME/local --with-libcrfsuite=$HOME/local

Make and install locally

    make
    make install

Finally, test the setup by following the instructions in the section
"Testing your installation" above.
