import re

from braces import expand_braces

def parameter_values(opt_string, default=None):
    """Return list of parameter values based on given (CLI) option
    string.

    Performs brace expansion ("a{1,2}" -> ["a1", "a2"]) and, for a
    restricted set of likely mathematical expressions, attempts eval()
    ("2**3" -> 8). Any value failing eval() is used as is.
    """

    if default is None:
        default = [1.0]    

    if opt_string is None:
        return default

    values = []
    for v in expand_braces(opt_string):
        try:
            values.append(str(eval(re.match(r'^([0-9.()*/+-]+)$', v).group(1))))
        except:
            values.append(v)
    return values

def expand_b_argument(b, tags):
    """Perform wildcard expansion for CLI label bias argument."""

    if b is None:
        return b
    
    expanded = []
    for arg in b.split(','):
        if ':' not in arg or '*' not in arg:
            expanded.append(arg)
        else:
            # TODO: graceful failures
            tag, weight = arg.split(':')
            assert tag.endswith('*'), 'b argument error'
            prefix = tag.rstrip('*')
            matching = _matching_tags(tags, prefix)
            expanded.append(','.join(['%s:%s' % (t, weight) for t in matching]))
    return ','.join(expanded)

def _matching_tags(tags, prefix):
    """Return set of tags in that begin with prefix (e.g. "B-")."""

    matching = set()
    for t in tags:
        if t.startswith(prefix):
            matching.add(t)
    return matching
