from docproc import Processor
from truecase import truecase, read_tc_data

CASING_NONE, CASING_LOWER, CASING_TRUECASE = 'none', 'lower', 'true'
DEFAULT_CASING=CASING_NONE
CASING_OPTIONS = [CASING_NONE, CASING_LOWER, CASING_TRUECASE]

def casing_processor(casing=CASING_NONE):
    if casing is None:
        casing = DEFAULT_CASING
    if casing == CASING_NONE:
        return []
    elif casing == CASING_LOWER:
        return [Lowercaser()]
    else:
        assert casing == CASING_TRUECASE
        return [Truecaser()]

class Truecaser(Processor):
    def start(self):
        Processor.start(self)
        read_tc_data()

    def _process(self, document):
        for sentence in document.sentences:
            sentence.orig_case_text = sentence.text
            sentence.text = truecase(sentence.text)
            assert sentence.text.lower() == sentence.orig_case_text.lower(), \
                'truecase() error: \n"%s"\n"%s"' % \
                (sentence.text.lower(), sentence.orig_case_text.lower())
            for token in sentence.tokens:
                tstart = token.start-sentence.base_offset
                tend = token.end-sentence.base_offset                
                token.orig_case_text = token.text
                token.text = sentence.text[tstart:tend]
            assert sentence.is_valid()
        document.orig_case_text = document.text
        document.text = ''.join(s.text for s in document.sentences)
        assert document.is_valid()

class Lowercaser(Processor):
    def _process(self, document):
        for sentence in document.sentences:
            sentence.orig_case_text = sentence.text
            sentence.text = sentence.text.lower()
            for token in sentence.tokens:
                token.orig_case_text = token.text
                token.text = token.text.lower()
            assert sentence.is_valid()
        document.orig_case_text = document.text
        document.text = document.text.lower()
        assert document.is_valid()
