import sys

import common

def argparser():
    ap = common.argparser(train_options=True)
    return ap

# callback for storing tags in process_documents()
def _tag_storer(tags):
    def tag_storer(document, tags=tags):
        tags |= document.get_tags()
    return tag_storer

def main(argv=None):
    if argv is None:
        argv = sys.argv

    options = argparser().parse_args(argv[1:])
    conf = common.get_configuration(options)

    processors = (common.input_processors(conf) + 
                  common.tagging_processors(conf) +
                  [common.NERsuiteTrainer.from_CLI_options(conf)])

    files = common.select_documents(options.files, True)

    tags = set()
    common.process_documents(files, processors, _tag_storer(tags))

    conf.tags = sorted(list(tags))
    conf.save()

    return 0
