import sys

import common

DEFAULT_PORT=8023

def argparser():
    ap = common.argparser(tag_options=True)
    ap.add_argument('-p', '--port', type=int, default=DEFAULT_PORT,
                    help='server port (with -S, default %d)' % DEFAULT_PORT)
    ap.add_argument('-S', '--server', default=False, action='store_true',
                    help='start tagging server')
    return ap

def main(argv=None):
    global options

    if argv is None:
        argv = sys.argv

    # Hack to make FILE args optional when running a server
    if '-S' in argv:
        argv.append('')

    options = argparser().parse_args(argv[1:])
    conf = common.get_configuration(options, revise=False)
    
    if conf.b:
        conf.b = common.expand_b_argument(conf.b, conf.tags)

    processors = (common.input_processors(conf) +
                  common.tagging_processors(conf) +
                  [common.NERsuiteTagger.from_CLI_options(conf)])

    if not options.server:
        processors.append(common.DocumentWriter.from_CLI_options(conf))
        files = common.select_documents(options.files)
        common.process_documents(files, processors)
    else:
        from common.server import serve
        def _serve(processors=None):
            common.server.serve(processors, port=options.port)
        common.run_processors(processors, _serve)

    return 0
