#!/bin/bash

set -u
set -e

# Given a directory with first-stage predictions in .nersuite format,
# an output directory, and a list of input files, create files with
# second-stage features in the output directory.

# Arguments specifying base feature set to use for the second-stage
# model.
FEATUREARG="" # nothing extra for now

# Commands to use for conversion, tag feature extraction from
# predictions, and adding those features to the base feature set.
PROG="./anatomytagger"
CONVERT="$PROG convert -F"
FEXTRACT="python common/tagfeatures.py"
FADD="python common/addfeatures.py -"

USAGESTR="Usage: $0 PREDDIR OUTDIR FILE [FILE [...]]"

if [ $# -lt 3 ]; then
    echo $USAGESTR
    exit 1
fi

PREDDIR=$1

if [ ! -d $PREDDIR ]; then
    echo "$PREDDIR is not a directory."
    exit 1
fi

OUTDIR=$2

if [ ! -d $OUTDIR ]; then
    echo "$OUTDIR is not a directory."
    exit 1
fi

shift 2
DATA=$@

DATATMP=`mktemp -d`

# Convert input data into NERsuite format, adding base features

$CONVERT -o $DATATMP $FEATUREARG $DATA

# Add features from first-stage predictions

for f in $DATATMP/*.nersuite; do
    p=$PREDDIR/`basename $f`
    if [ ! -e $p ]; then
	echo "missing predictions: $p"
	exit 1
    fi
    $FEXTRACT $p | $FADD $f > $OUTDIR/`basename $f`
done

rm -rf DATATMP
