#!/bin/bash

set -u
set -e

DATASET=data/AnatEM
TRAINSET=$DATASET/standoff/train/*.ann
TESTSET=$DATASET/standoff/devel/*.ann

LEARNPARAM="-c2 0.001953"
TAGPARAM="-b B-Anatomical_system:2.0,B-Cancer:2.0,B-Cell:2.0,B-Cellular_component:2.0,B-Developing_anatomical_structure:2.0,B-Equiv:2.0,B-frag:2.0,B-Immaterial_anatomical_entity:2.0,B-Multi-tissue_structure:2.0,B-Organ:2.0,B-Organism_subdivision:2.0,B-Organism_substance:2.0,B-Part-of:2.0,B-Pathological:2.0,B-Pathological_formation:2.0,B-Plant:2.0,B-Processed:2.0,B-Tissue:2.0"

PROG="./anatomytagger"
CVCMD="$PROG eval -x 10"
LEARNCMD="$PROG learn"
PREDCMD="$PROG tag"
CONVERTCMD="$PROG convert"
EVALCMD="./evalbio.pl -a 0 -r 0"

# Learn first-stage model

S1MODEL=`mktemp -p . -t stage1.model.XXX`

$LEARNCMD $LEARNPARAM -m $S1MODEL $TRAINSET

# Run cross-validation to create second-stage training data.

CVTMP=`mktemp -d`

$CVCMD $LEARNPARAM $TAGPARAM -o $CVTMP $TRAINSET

S2TMP=`mktemp -d`

./featurize-stage2.sh $CVTMP $S2TMP $TRAINSET

rm -rf $CVTMP

# Learn second-stage model

S2MODEL=`mktemp -p . -t stage1.model.XXX`

$LEARNCMD $LEARNPARAM -m $S2MODEL $S2TMP/*.nersuite

rm -rf $S2TMP

# Run stage 1 predictions on test data

S1PREDTMP=`mktemp -d`

$PREDCMD $TAGPARAM -m $S1MODEL -o $S1PREDTMP $TESTSET

# Create stage 2 data and run stage 2 predictions

S2TESTDIR=`mktemp -d`

./featurize-stage2.sh $S1PREDTMP $S2TESTDIR $TESTSET

S2PREDTMP=`mktemp -d`

$PREDCMD $TAGPARAM -m $S2MODEL -o $S2PREDTMP $S2TESTDIR/*.nersuite

rm -rf $S2TESTDIR

# Evaluate

GOLDTMP=`mktemp -d`

$CONVERTCMD -o $GOLDTMP $TESTSET

echo "Stage 1:"
$EVALCMD <(cat $S1PREDTMP/*.nersuite) <(cat $GOLDTMP/*.nersuite)

echo
echo "Stage 2:"
$EVALCMD <(cat $S2PREDTMP/*.nersuite) <(cat $GOLDTMP/*.nersuite)

# Final cleanup

rm -rf $GOLDTMP
rm -rf $S1PREDTMP
rm -rf $S2PREDTMP

# Note: if you want to keep the models, comment out the following lines

rm -rf $S1MODEL
rm -rf $S2MODEL
