#!/bin/bash

set -u
set -e

USAGESTR="Usage: $0 TRAINDIR TESTDIR"

if [ $# -lt 2 -o $# -gt 2 ]; then
    echo $USAGESTR
    exit 1
fi

TRAINDIR=$1
TESTDIR=$2

TRAINSET=$TRAINDIR/*.ann
TESTSET=$TESTDIR/*.ann

S1_C2_VALUE="-c2 0.001953"
S1_B_VALUE="-b B-Anatomical_system:2.0,B-Cancer:2.0,B-Cell:2.0,B-Cellular_component:2.0,B-Developing_anatomical_structure:2.0,B-Equiv:2.0,B-frag:2.0,B-Immaterial_anatomical_entity:2.0,B-Multi-tissue_structure:2.0,B-Organ:2.0,B-Organism_subdivision:2.0,B-Organism_substance:2.0,B-Part-of:2.0,B-Pathological:2.0,B-Pathological_formation:2.0,B-Plant:2.0,B-Processed:2.0,B-Tissue:2.0"

# 2nd-stage parameter values to test
S2_C2_VALUES='2**{5..-10..-1}'
S2_B_VALUES='B-*:{-5..5..0.5}'

PROG="./anatomytagger"
CVCMD="$PROG eval -x 10"
LEARNCMD="$PROG learn"
PREDCMD="$PROG tag"
CONVERTCMD="$PROG convert"
EVALCMD="./evalbio.pl -a 0 -r 0"

# Learn first-stage model

S1MODEL=`mktemp -p . -t stage1.model.XXX`

$LEARNCMD $S1_C2_VALUE -m $S1MODEL $TRAINSET

# Run stage 1 predictions on test data and create stage 2 test data

S1PREDTMP=`mktemp -d`

$PREDCMD $S1_B_VALUE -m $S1MODEL -o $S1PREDTMP $TESTSET

rm -rf $S1MODEL

S2TESTDIR=`mktemp -d`

./featurize-stage2.sh $S1PREDTMP $S2TESTDIR $TESTSET

# Convert gold and evaluate first-stage model (for reference, not a
# required step)

GOLDTMP=`mktemp -d`

$CONVERTCMD -o $GOLDTMP $TESTSET

echo "Stage 1:"
$EVALCMD <(cat $S1PREDTMP/*.nersuite) <(cat $GOLDTMP/*.nersuite)

rm -rf $S1PREDTMP
rm -rf $GOLDTMP

# Run cross-validation to create stage 2 training data.

CVTMP=`mktemp -d`

$CVCMD $S1_C2_VALUE $S1_B_VALUE -o $CVTMP $TRAINSET

S2TRAINDIR=`mktemp -d`

./featurize-stage2.sh $CVTMP $S2TRAINDIR $TRAINSET

rm -rf $CVTMP

# run stage 2 parameter selection

$PROG eval -c2 $S2_C2_VALUES -b $S2_B_VALUES $S2TRAINDIR/*.nersuite -t $S2TESTDIR/*.nersuite

rm -rf $S2TRAINDIR
rm -rf $S2TESTDIR
