#!/bin/bash

# Convert MetaMap output from its prolog format into XML using MetaMap
# print (http://metamap.nlm.nih.gov/#MetaMapPrint). Prior to
# conversion, the script splits up its input into files by source
# document IDs.

# This script was written for processing the MetaMapped Medline data
# (http://skr.nlm.nih.gov/resource/MetaMappedBaselineInfo.shtml) in a
# way that would minimize data loss from MetaMap print crashes and
# operate in an environment where the large dataset must be stored on
# a server separate from where it is processed. If these issues are
# not a concern, it will likely be easier to use MetaMap print
# directly.

MM_PRINT=~/tools/MetaMap-print/public_mm/bin/mm_print12

# Input MetaMapped Medline (or similar) packages
DATA_IN=data/MetaMapped_Medline
# Location for processed packages
DATA_OUT=data/MetaMapped_Medline-XML/

# Directory for data split into one-document-per-file format
SPLIT_DIR=split

for i in `seq -w 01 99` `seq -w 100 684`; do 
    f=text.out_$i.gz;
    echo "$0: processing" `basename $f`

    scp ${DATA_IN}/$f .
    gunzip -f $f
    rm -rf $SPLIT_DIR
    mkdir $SPLIT_DIR
    cat ${f%.gz} | python splitmm.py

    for x in $SPLIT_DIR/*; do
        # the --XMLf1 option specifies XML output with a single XML
        # document (the "1") for an entire input file.
	$MM_PRINT --XMLf1 $x > $x.xml
	rm $x
    done

    of=${f%.gz}-xml.tar.gz
    tar czf $of $SPLIT_DIR    
    scp $of $DATA_OUT
    rm -rf ${f%.gz} $of $SPLIT_DIR
done
