#!/bin/bash

# Convert MetaMap output XML into a simple TSV format using
# mmxml2tsv.py.

# This script was written for processing the MetaMapped Medline data
# (http://skr.nlm.nih.gov/resource/MetaMappedBaselineInfo.shtml) in an
# environment where the large dataset must be stored on a server
# separate from where it is processed. If these issues are not a
# concern, it will likely be easier to use mmxml2tsv.py directly.

# Input MetaMap XML packages
DATA_IN=data/MetaMapped_Medline-XML/
# Location for processed packages
DATA_OUT=data/MetaMapped_Medline-TSV/

# Directory for data split into one-document-per-file format
# (must match same in mm2xml.sh)
SPLIT_DIR=split

for i in `seq -w 01 99` `seq -w 100 684`; do 
    f=text.out_${i}-xml.tar.gz; 
    echo "$0: processing" `basename $f`

    scp $DATA_IN/$f .
    tar xzf $f
    rm $f

    of=${f/-xml.tar.gz/.tsv}
    rm -rf $of

    for x in $SPLIT_DIR/*.xml; do
	# perl to remove non-XML header output by MetaMap print
	perl -pe '$inXML=1 if(/^<\?xml/); $_="" unless($inXML);' $x | python mmxml2tsv.py - >> $of
    done

    gzip $of
    scp $of.gz $DATA_OUT
    rm -rf $of.gz $SPLIT_DIR
done
