#!/usr/bin/env python

# Split MetaMap output into smaller parts.

# This is a special-purpose script written to support processing of
# MetaMapped Medline data, available for download from
# http://skr.nlm.nih.gov/resource/MetaMappedBaselineInfo.shtml.

import sys
import re
import os

OUTPUT_DIRECTORY = 'split'

BLOCK_START_RE = re.compile(r'^args\(')
PMID_RE = re.compile(r"utterance\('(\d+)\.")

DOC_LIMIT = 1

def process_block(block):
    if len(block) == 0:
        return False
    
    PMID = None
    for l in block:
        m = PMID_RE.match(l)
        if m:
            PMID = m.group(1)
            break
    if PMID is None:
        print >> sys.stderr, "Error: failed to find PMID in block:\n", ''.join(block)
        return False

    with open(os.path.join(OUTPUT_DIRECTORY, PMID), 'wt') as o:
        for l in block:
            o.write(l)

    return True

def main(argv=None):
    if argv is None:
        argv = sys.argv

    doc_count = 0
    current_block = []
    for l in sys.stdin:
        if BLOCK_START_RE.match(l):
            doc_count += 1

        if doc_count >= DOC_LIMIT:
            process_block(current_block)
            current_block = []
            doc_count = 0

        current_block.append(l)

    process_block(current_block)

    return 0

if __name__ == "__main__":
    sys.exit(main(sys.argv))
